package meerkat.util;

import meerkat.crypto.DigitalSignature;
import meerkat.protobuf.BulletinBoardAPI.BulletinBoardMessage;
import meerkat.protobuf.BulletinBoardAPI.UnsignedBulletinBoardMessage;
import meerkat.protobuf.Crypto;

import java.security.SignatureException;
import java.util.*;

/**
 * Created by Arbel Deutsch Peled on 16-Feb-16.
 */
public class BulletinBoardUtils {

    /**
     * Creates BulletinBoardMessage with UnsignedBulletinBoardMessage and its signature
     *         signed by all given DigitalSignatures
     *
     * @param unsignedMessage BasicMessage
     * @param signer collection of DigitalSignature which will sign the
     *                UnsignedBulletinBoardMessage message
     * @return BulletinBoardMessage
     */
    public static BulletinBoardMessage signBulletinBoardMessage(UnsignedBulletinBoardMessage unsignedMessage,
                                                                DigitalSignature signer) throws SignatureException {
        BulletinBoardMessage.Builder bulletinBoardMessage = BulletinBoardMessage.newBuilder();
        bulletinBoardMessage.setMsg(unsignedMessage);
        signer.updateContent(unsignedMessage);
        Crypto.Signature signature = signer.sign();
        bulletinBoardMessage.addSig(signature);
        return bulletinBoardMessage.build();
    }

    /**
     * Gets the latest message from collection of messages (BulletinBoardMessage) based on timestamp
     * @param messages List of BulletinBoardMessage
     * @return BulletinBoardMessage
     */
    public static BulletinBoardMessage getLatestMessage(Collection<BulletinBoardMessage> messages){
        return Collections.max(messages, new Comparator<BulletinBoardMessage>() {
            @Override
            public int compare(BulletinBoardMessage o1, BulletinBoardMessage o2) {
                TimestampComparator comparator = new TimestampComparator();
                return comparator.compare(o1.getMsg().getTimestamp(), o2.getMsg().getTimestamp());
            }
        });
    }

    /**
     * Searches the tags in the message for one that begins with given prefix
     * @param message is the message to search
     * @param prefix is the given prefix
     * @return the tag without the prefix, if found, or null if not found
     */
    public static String findTagWithPrefix(BulletinBoardMessage message, String prefix) {
        for (String tag : message.getMsg().getTagList()){
            if (tag.startsWith(prefix)) {
                return tag.substring(prefix.length());
            }
        }

        return null;

    }



    /**
     * Gets list of tags values from given messages (tagName values)
     * @param messages list of messages from which we want to retrieve specific tag values
     * @return List<String>
     */
    public static List<String> GetListOfTags(List<BulletinBoardMessage>  messages, String tagName) {
        List<String> tagsValues = new ArrayList<>(messages.size());
        for ( int i = 0 ; i < messages.size() ; i++ ){
            BulletinBoardMessage message = messages.get(i);
            tagsValues.add(findTagWithPrefix(message, tagName));
        }
        return tagsValues;
    }

    /**
     * Searches the tags in a message for tags that do not contain a given list of prefixes
     * @param message is the message to search
     * @param prefixes is the list of prefixes
     * @return a list of the tags that do *not* contain any of the given prefixes
     */
    public static List<String> removePrefixTags(BulletinBoardMessage message, Iterable<String> prefixes) {

        if (prefixes == null)
            return message.getMsg().getTagList();

        List<String> result = new LinkedList<>();

        for (String tag : message.getMsg().getTagList()){

            boolean found = false;

            for (String prefix : prefixes){
                if (tag.startsWith(prefix)){
                    found = true;
                    break;
                }
            }

            if (!found) {
                result.add(tag);
            }

        }

        return result;

    }

    /**
     * This method creates a Timestamp Protobuf from a time specification
     * @param timeInMillis is the time to encode since the Epoch time in milliseconds
     * @return a Timestamp Protobuf encoding of the given time
     */
    public static com.google.protobuf.Timestamp toTimestampProto(long timeInMillis) {

        return com.google.protobuf.Timestamp.newBuilder()
                .setSeconds(timeInMillis / 1000)
                .setNanos((int) ((timeInMillis % 1000) * 1000000))
                .build();

    }

    /**
     * This method creates a Timestamp Protobuf from the current system time
     * @return a Timestamp Protobuf encoding of the current system time
     */
    public static com.google.protobuf.Timestamp getCurrentTimestampProto() {

        return toTimestampProto(System.currentTimeMillis());

    }

    /**
     * This method converts an SQL Timestamp object into a Protobuf Timestamp object
     * @param sqlTimestamp is the SQL Timestamp
     * @return an equivalent Protobuf Timestamp
     */
    public static com.google.protobuf.Timestamp toTimestampProto(java.sql.Timestamp sqlTimestamp) {
        return toTimestampProto(sqlTimestamp.getTime());
    }

    /**
     * This method converts a Protobuf Timestamp object into an SQL Timestamp object
     * @param protoTimestamp is the Protobuf Timestamp
     * @return an equivalent SQL Timestamp
     */
    public static java.sql.Timestamp toSQLTimestamp(com.google.protobuf.Timestamp protoTimestamp) {
        return new java.sql.Timestamp(protoTimestamp.getSeconds() * 1000 + protoTimestamp.getNanos() / 1000000);
    }

}
