package meerkat;

import com.google.common.util.concurrent.FutureCallback;
import meerkat.bulletinboard.SubscriptionAsyncBulletinBoardClient;
import meerkat.crypto.DigitalSignature;
import meerkat.protobuf.BulletinBoardAPI.UnsignedBulletinBoardMessage;
import meerkat.protobuf.VoterRegistry.VoterID;
import meerkat.protobuf.VoterRegistry.VoterInfo;
import meerkat.protobuf.VoterRegistry.VoterRegistryMessage;
import meerkat.registry.AsyncRegistryCallbacks.GetGroupsCallback;
import meerkat.registry.AsyncRegistryCallbacks.GetVoterCallback;
import meerkat.registry.AsyncRegistryCallbacks.HasVotedCallback;
import meerkat.registry.MessageCollectionUtils;
import meerkat.registry.RegistryTags;
import meerkat.util.BulletinBoardUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.security.SignatureException;
import java.util.ArrayList;
import java.util.List;

import static meerkat.util.BulletinBoardUtils.signBulletinBoardMessage;


//  TODO : add ability to use DB of certificates and add logging

/**
 * Created by Vladimir Eliezer Tokarev on 1/8/2016.
 * Gives the ability to synchronously manage voters information
 */
public class AsyncRegistry implements VoterRegistry{

    final Logger logger = LoggerFactory.getLogger(getClass());

    protected DigitalSignature signer;
    protected SubscriptionAsyncBulletinBoardClient cachedBulletinBoardClient;

    @Override
    public void init(DigitalSignature signer, SubscriptionAsyncBulletinBoardClient communicator) {
        logger.debug("Initialized the AsyncRegistry");
        this.signer = signer;
        this.cachedBulletinBoardClient = communicator;
    }

    @Override
    public void addVoter(VoterInfo voterInfo, FutureCallback<Boolean> callback) throws SignatureException {
        logger.info("Adding voter : {0}", RegistryTags.ID_TAG);
        UnsignedBulletinBoardMessage basicMessage =
                UnsignedBulletinBoardMessage.newBuilder()
                        .addTag(RegistryTags.ID_TAG + voterInfo.getId().getId())
                        .addTag(RegistryTags.VOTER_ENTRY_TAG)
                        .setData(voterInfo.toByteString())
                        .setTimestamp(BulletinBoardUtils.getCurrentTimestampProto())
                        .build();


        cachedBulletinBoardClient.postMessage(signBulletinBoardMessage(basicMessage, signer), callback);
    }

    @Override
    public void setVoterGroups(VoterRegistryMessage voterRegistryMessage, FutureCallback<Boolean> callback) throws SignatureException, IOException {
        UnsignedBulletinBoardMessage.Builder basicMessage =
                UnsignedBulletinBoardMessage.newBuilder()
                        .addTag(RegistryTags.ID_TAG + voterRegistryMessage.getVoterID().getId())
                        .addTag(RegistryTags.ADD_TO_GROUP_TAG)
                        .setData(voterRegistryMessage.toByteString())
                        .setTimestamp(BulletinBoardUtils.getCurrentTimestampProto());

        cachedBulletinBoardClient.postMessage(signBulletinBoardMessage(basicMessage.build(), signer), callback);
    }

    @Override
    public void setVoted(VoterID voterId, FutureCallback<Boolean> callback) throws SignatureException {
        UnsignedBulletinBoardMessage basicMessage =
                UnsignedBulletinBoardMessage.newBuilder()
                        .addTag(RegistryTags.ID_TAG + voterId.getId())
                        .addTag(RegistryTags.VOTE_ACTION_TAG)
                        .setTimestamp(BulletinBoardUtils.getCurrentTimestampProto())
                        .build();

        cachedBulletinBoardClient.postMessage(signBulletinBoardMessage(basicMessage, signer), callback);
    }

    @Override
    public void getGroups(VoterID voterID, FutureCallback<List<meerkat.protobuf.VoterRegistry.GroupID>> callback) {
        List<String> GroupsActionsTags = new ArrayList<String>(2) {{
           add(RegistryTags.ADD_TO_GROUP_TAG);
           add(RegistryTags.ID_TAG + voterID.getId());
        }};

        cachedBulletinBoardClient.readMessages(MessageCollectionUtils.generateFiltersFromTags(GroupsActionsTags),
                new GetGroupsCallback(callback));
    }

    @Override
    public void getVoter(VoterID voterID, FutureCallback<VoterInfo> callback) {
        List<String> addVoterTags = new ArrayList<String>(2) {{
            add(RegistryTags.ID_TAG + voterID.getId());
            add(RegistryTags.VOTER_ENTRY_TAG);
        }};

        cachedBulletinBoardClient.readMessages(MessageCollectionUtils.generateFiltersFromTags(addVoterTags),
                new GetVoterCallback(callback));
    }

    @Override
    public void hasVoted(VoterID voterId, FutureCallback<Boolean> callBack) {
        List<String> setVotedTags = new ArrayList<String>(2) {{
            add(RegistryTags.ID_TAG + voterId.getId());
            add(RegistryTags.VOTE_ACTION_TAG);
        }};

        cachedBulletinBoardClient.readMessages(MessageCollectionUtils.generateFiltersFromTags(setVotedTags),
                new HasVotedCallback(callBack));
    }
}
